@@ -1,4 +1,4 @@
-/*$Id: config.h,v 1.88.2.3 2001/01/08 08:11:11 guenther Exp $*/
+/*$Id: config.h,v 1.88.2.4 2001/07/15 09:26:57 guenther Exp $*/
 
 /*#define sMAILBOX_SEPARATOR	"\1\1\1\1\n"	/* sTART- and eNDing separ.  */
 /*#define eMAILBOX_SEPARATOR	"\1\1\1\1\n"	/* uncomment (one or both)
@@ -29,7 +29,7 @@
  * completion of the /etc/procmailrc file (future versions of procmail are
  * expected to provide better runtime configuration control).  The autoconf
  * process attempts to determine reasonable values for these versions of PATH
- * and setts the defSPATH and defPATH variables accordingly.  If you want to
+ * and sets the defSPATH and defPATH variables accordingly.  If you want to
  * override those settings you should uncomment and possibly change the
  * DEFSPATH and DEFPATH defines below
  */
@@ -78,8 +78,8 @@
 /*#define RESTRICT_EXEC 100	/* uncomment to prevent users with uids equal
 				   or higher than RESTRICT_EXEC from
 	executing programs from within their .procmailrc files (this
-	restriction does not apply to /etc/procmailrc and /etc/procmailrcs
-	files) */
+	restriction does not apply to the /etc/procmailrc and
+	/etc/procmailrcs files) */
 
 /*#define NO_NFS_ATIME_HACK	/* uncomment if you're definitely not using
 				   NFS mounted filesystems and can't afford
@@ -146,6 +146,8 @@
 #define BLKSIZ		1024
 #define STDBUF		128
 #endif /* SMALLHEAP */
+#define MINlogbuf	81			       /* fit an entire line */
+#define MAXlogbuf	1000		       /* in case someone abuses LOG */
 #define FAKE_FIELD	">From "
 #define RETRYunique	8	   /* # of tries at making a unique filename */
 #define BOGUSprefix	"BOGUS."	     /* prepended to bogus mailboxes */
@@ -193,7 +195,8 @@ MMGR)\
 #define DEFcomsat	"no"		/* when an rcfile has been specified */
 
 #define BinSh		"/bin/sh"
-#define RootDir		"/"
+#define ROOT_DIR	"/"
+#define DEAD_LETTER	"/tmp/dead.letter"    /* $ORGMAIL if no passwd entry */
 #define DevNull		"/dev/null"
 #define NICE_RANGE	39			  /* maximal nice difference */
 #define chCURDIR	'.'			    /* the current directory */
@@ -204,6 +207,7 @@ MMGR)\
 #define MAILDIRcur	"/cur"
 #define MAILDIRnew	"/new"
 #define MAILDIRLEN	STRLEN(MAILDIRnew)
+#define MAILDIRretries	5	   /* retries on obtaining a unique filename */
 
 #define EOFName		" \t\n#`'\");"
 
