@@ -1,12 +1,12 @@
 /************************************************************************
  *	Some routine common to procmail, formail and lockfile		*
  *									*
- *	Copyright (c) 1993-1994, S.R. van den Berg, The Netherlands	*
+ *	Copyright (c) 1993-1999, S.R. van den Berg, The Netherlands	*
  *	#include "../README"						*
  ************************************************************************/
 #ifdef RCS
 static /*const*/char rcsid[]=
- "$Id: acommon.c,v 1.2 1994/06/28 16:55:56 berg Exp $";
+ "$Id: acommon.c,v 1.10 1999/11/02 03:50:59 guenther Exp $";
 #endif
 #include "includes.h"
 #include "acommon.h"
@@ -19,26 +19,28 @@ const char*hostname P((void))
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN	64
 #endif
-  static char name[MAXHOSTNAMELEN];
-  gethostname(name,MAXHOSTNAMELEN);name[MAXHOSTNAMELEN-1]='\0';
+  static char name[MAXHOSTNAMELEN]="";
+  if(!name[0])
+     gethostname(name,MAXHOSTNAMELEN),name[MAXHOSTNAMELEN-1]='\0';
 #else
-  struct utsname names;static char*name;
-  if(name)
-     free(name);
-  Uname(&names);
-  if(!(name=malloc(strlen(names.nodename)+1)))
-     return "";		      /* can happen when called from within lockfile */
-  strcpy(name,names.nodename);
+  static char*name=0;
+  if(!name)
+   { struct utsname names;
+     Uname(&names);
+     if(!(name=malloc(strlen(names.nodename)+1)))
+	return "";	      /* can happen when called from within lockfile */
+     strcpy(name,names.nodename);
+   }
 #endif
   return name;
 }
 
 char*ultoan(val,dest)unsigned long val;char*dest;     /* convert to a number */
-{ register i;				     /* within the set [0-9A-Za-z-_] */
+{ register int i;			     /* within the set [A-Za-z0-9-_] */
   do
    { i=val&0x3f;			   /* collating sequence dependency! */
-     *dest++=i+(i<10?'0':i<10+26?'A'-10:i<10+26+26?'a'-10-26:
-      i==10+26+26?'-'-10-26-26:'_'-10-26-27);
+     *dest++=i+(i<26?'A':i<26+26?'a'-26:i<26+26+10?'0'-26-26:
+      i==26+26+10?'-'-26-26-10:'_'-26-26-11);
    }
   while(val>>=6);
   *dest='\0';
