@@ -1,14 +1,14 @@
-/*$Id: common.h,v 1.10 1994/05/26 14:12:28 berg Exp $*/
+/*$Id: common.h,v 1.10.4.2 2001/07/15 09:27:09 guenther Exp $*/
 
 void
- shexec P((const char*const*argv)),
+ shexec P((const char*const*argv)) __attribute__((noreturn)),
  detab P((char*p)),
  ultstr P((int minwidth,unsigned long val,char*dest));
 char
  *skpspace P((const char*chp));
 int
  waitfor Q((const pid_t pid)),
- strnIcmp P((const char*a,const char*b,size_t l));
+ strnIcmp Q((const char*a,const char*b,size_t l));
 
 #ifdef NOstrcspn
 int
