@@ -1,4 +1,4 @@
-/*$Id: pipes.h,v 1.9 1994/05/26 14:13:20 berg Exp $*/
+/*$Id: pipes.h,v 1.10.4.1 2001/07/15 09:27:28 guenther Exp $*/
 
 void
  inittmout P((const char*const progname)),
@@ -8,7 +8,7 @@ void
 int
  pipthrough P((char*line,char*source,const long len));
 long
- pipin P((char*const line,char*source,long len));
+ pipin P((char*const line,char*source,long len,const int asgnlastf));
 char*
  readdyn P((char*bf,long*const filled)),
  *fromprog Q((char*name,char*const dest,size_t max));
@@ -24,5 +24,5 @@ extern const char exitcode[];
 extern int setxit;
 extern pid_t pidchild;
 extern volatile time_t alrmtime;
-extern volatile toutflag;
+extern volatile int toutflag;
 extern int pipw;
