@@ -14,7 +14,7 @@
  ************************************************************************/
 #ifdef RCS
 static /*const*/char rcsid[]=
- "$Id: procmail.c,v 1.152.2.9 2001/01/08 07:27:03 guenther Exp $";
+ "$Id: procmail.c,v 1.152.2.11 2001/07/16 04:26:58 guenther Exp $";
 #endif
 #include "../patchlevel.h"
 #include "procmail.h"
@@ -35,6 +35,7 @@ static /*const*/char rcsid[]=
 #include "lastdirsep.h"
 #include "authenticate.h"
 #include "foldinfo.h"
+#include "comsat.h"
 
 static const char*const nullp,From_[]=FROM,exflags[]=RECFLAGS,
  drcfile[]="Rcfile:",pmusage[]=PM_USAGE,*etcrc=ETCRC,
@@ -52,12 +53,12 @@ const char shell[]="SHELL",lockfile[]="LOCKFILE",newline[]="\n",binsh[]=BinSh,
  insufprivs[]="Insufficient privileges\n",
  exceededlb[]="Exceeded LINEBUF\n",errwwriting[]="Error while writing to";
 char*Stdout;
-int retval=EX_CANTCREAT,retvl2=EXIT_SUCCESS,sh,pwait,lcking,rcstate,rc= -1,
+int retval=EX_CANTCREAT,retvl2=EXIT_SUCCESS,sh,pwait,rcstate,rc= -1,
  ignwerr,lexitcode=EXIT_SUCCESS,asgnlastf,crestarg,skiprc,savstdout,berkeley,
  mailfilter,erestrict,ifdepth;			       /* depth of outermost */
 struct dyna_long ifstack;	      /* brace in this rcfile, and the stack */
 size_t linebuf=mx(DEFlinebuf,1024/*STRLEN(systm_mbox)<<1*/);
-volatile int nextexit;			       /* if termination is imminent */
+volatile int nextexit,lcking;		       /* if termination is imminent */
 pid_t thepid;
 long filled,lastscore;	       /* the length of the mail, and the last score */
 char*themail,*thebody;			    /* the head and body of the mail */
@@ -99,12 +100,12 @@ int main(argc,argv)const char*const argv[];
   newid();
   ;{ int presenviron,Deliverymode,override;char*fromwhom=0;
      const char*idhint=0;gid_t egid=getegid();
-     Deliverymode=mailfilter=override=0;
+     presenviron=Deliverymode=mailfilter=override=0;
      Openlog(procmailn,LOG_PID,LOG_MAIL);		  /* for the syslogd */
      if(argc)			       /* sanity check, any argument at all? */
       { Deliverymode=strncmp(lastdirsep(argv0=argv[0]),procmailn,
 	 STRLEN(procmailn));
-	for(presenviron=argc=0;(chp2=(char*)argv[++argc])&&*chp2=='-';)
+	for(argc=0;(chp2=(char*)argv[++argc])&&*chp2=='-';)
 	   for(;;)				       /* processing options */
 	    { switch(*++chp2)
 	       { case VERSIONOPT:elog(procmailn);elog(VERSION);
@@ -169,7 +170,7 @@ setarg:		       *argv1=chp2;restargv=argv1;crestarg=1;
 		       goto last_option;
 		     }
 		 case '-':
-		    if(!*chp2)
+		    if(!*++chp2)
 		     { argc++;
 		       goto last_option;
 		     }
@@ -358,6 +359,7 @@ no_from:       { tstamp=0;	   /* no existing From_, so nothing to stamp */
 	    }
 	   while(chp=(char*)argv[argc]);
 	gargv=argv+argc;suppmunreadable=verbose; /* save it for nextrcfile() */
+	setcomsat(empty);			  /* turn on biff by default */
 	if(Deliverymode)	/* try recipient without changing case first */
 	 { if(!(pass=auth_finduser(chp2,-1)))	    /* chp2 is the recipient */
 	    { static const char unkuser[]="Unknown user";
@@ -377,7 +379,7 @@ no_from:       { tstamp=0;	   /* no existing From_, so nothing to stamp */
 	   if(presenviron)		      /* preserving the environment? */
 	      etcrc=0;				    /* don't read etcrc then */
 	   if(suppmunreadable)			     /* command-line rcfile? */
-	      etcrc=0,scomsat=DEFcomsat;	  /* forget etcrc and comsat */
+	      etcrc=0,setcomsat(DEFcomsat);	  /* forget etcrc and comsat */
 	   if(mailfilter)
 	    { if(!suppmunreadable)
 	       { nlog("Missing rcfile\n");
@@ -456,10 +458,11 @@ Setuser: { gid=auth_whatgid(pass);uid=auth_whatuid(pass);
 	  /*
 	   *	to prevent security holes, drop any privileges now
 	   */
-	 { setdef(lgname,buf);setdef(home,RootDir);setdef(shell,binsh);
-	   setdef(orgmail,"/tmp/dead.letter");setids();
+	 { setdef(lgname,buf);setdef(home,ROOT_DIR);setdef(shell,binsh);
+	   setdef(orgmail,DEAD_LETTER);setids();
 	 }
 	endpwent();auth_freeid(spassinvk);
+	setlgcs(tgetenv(lgname));	  /* make sure sendcomsat has a copy */
       }
      if(!presenviron||!mailfilter)	  /* by default override environment */
       { setdef(host,hostname());sputenv(lastfolder);sputenv(exitcode);
@@ -768,10 +771,8 @@ nolock:			{ nlog("Couldn't determine implicit lockfile from");
 		       filled=startchar-themail,readmail(0,0L),succeed=!pipw;
 		  }
 		 else if(Stdout)		  /* capturing stdout again? */
-		  { if(!pipthrough(buf,startchar,tobesent))
-		       succeed=1,postStdout();	  /* only parse if no errors */
-		  }
-		 else if(!pipin(buf,startchar,tobesent))  /* regular program */
+		    succeed=!pipthrough(buf,startchar,tobesent);
+		 else if(!pipin(buf,startchar,tobesent,1))/* regular program */
 		    if(succeed=1,flags[CONTINUE])
 		       goto logsetlsucc;
 		    else
@@ -1005,7 +1006,7 @@ drop: { *ep= *--emax;*emax=0;				/* copy from the end */
      for(pp=(const char*const*)environ;pp!=ep;pp++)	 /* duplicate entry? */
 	if(!strncmp(*ep,*pp,len))
 	   goto drop;
-     for(pp=ld_;p= *pp;pp++)	      /* does it start with LD_ or similar? */
+     for(pp=ld_;p= *pp;pp++)	       /* does it start with LD_ or similar? */
 	if(!strncmp(*ep,p,strlen(p)))
 	   goto drop;
      ep++;
